; (C) Copyright Motek Information Systems, Inc 1993-1994
; All rights reserved
; You may modify or redistribute this file in any way you like,
; provided that you maintain the copyright notice above.
;
; This program requires Paradox for DOS 4.0 or greater.  It will
; not work with Paradox for Windows.  In order to print Code 39
; barcodes from anywhere in Windows, the combination of a Code 39
; True Type font (many are available shareware) and the program
; "FontMonger" (which allows non-proportional scaling of TrueType
; fonts) is quite powerful.
;
; Feedback is welcomed.  Send comments to chi@netcom.com


; THEORY OF OPERATION
;
; Calculating the strings that will generate the desired barcode
; is, unfortunately, a little slower in PAL than is desirable.
; To speed up the printing of many barcodes of the same size (this
; application was designed to print twenty to a page and was eventually
; redone in C), the work has been split into two parts.
;
; The function InitCode39(pitch, height) does the precomputation.  It
; calculates strings for each of the characters in the Code 39 alphabet
; based on the two parameters.
;
; "pitch" is a measure related to the width of the bars.  The thick
; bars will be 5 times the size of pitch and the thin ones will be 2
; times that size.  The measurements are given in pixels, which on
; an HP Laserjet II or III, come roughly 300 to the inch.
;
; "height" is the height of the barcode, again, in pixels
;
; Once InitCode39() has been called, barcodes can be generated by
; calling BuildBarcode(src) and sending the return value to the
; printer.  A sample printing PROC is also provided.
;
; If you want to change the pitch or height of the barcode, InitCode39()
; will have to be called again; otherwise, only BuildBarcode() is
; required for further barcodes.

; PRINTERS
;
; You should be able to print barcodes for different printers
; by changing the printer commands in the following procedures 
; to match your printer.  All else stays the same.
;
; These printer commands will work with most HP LaserJet models, being
; basically PCL 3.

; Rect() returns a string that, when sent to the printer, prints
; a rectangle at the current point with width dx and height dy.

PROC Rect(dx, dy)
   RETURN "\027*c" + STRVAL(dx) + "a" + STRVAL(dy) + "b0P"
ENDPROC

; MoveX() returns a string that, when sent to the printer, moves
; the current point along the x axis by dx.

PROC MoveX(dx)
   IF (dx >= 0) THEN
      RETURN "\027*p+" + STRVAL(dx) + "X"
   ELSE
      RETURN "\027*p" + STRVAL(dx) + "X"
   ENDIF
ENDPROC

; These two procedures aren't used, but may be helpful

; MoveToXY() returns a string that, when sent to the printer, moves the
; current point to absolute position x,y on the page.

PROC MoveToXY(x, y)
   RETURN "\027*p" + STRVAL(x) + "x" + STRVAL(y) + "Y"
ENDPROC

; MoveY() is like MoveX(), except that the point is moved along the y axis
; by dy.

PROC MoveY(dy)
   IF (dy >= 0) THEN
      RETURN "\027*p+" + STRVAL(dx) + "Y"
   ELSE
      RETURN "\027*p+" + STRVAL(dx) + "Y"
   ENDIF
ENDPROC

; Here ends the functions which are printer-specific

PROC Decode(astring, pos, width)
   IF (SUBSTR(astring, pos, 1) = "0") THEN
      RETURN int(2*width + 0.5);
   ELSE
      RETURN int(5*width + 0.5);
   ENDIF
ENDPROC

PROC CalcCode(template, width, height)
   PRIVATE
      tmp,
      w

   w = Decode(template, 1, width)
   tmp = Rect(w, height) + MoveX(w + Decode(template, 6, width))
   
   w = Decode(template, 2, width)
   tmp = tmp + Rect(w, height) + MoveX(w + Decode(template, 7, width))
   
   w = Decode(template, 3, width)     
   tmp = tmp + Rect(w, height) + MoveX(w + Decode(template, 8, width))
   
   w = Decode(template, 4, width)     
   tmp = tmp + Rect(w, height) + MoveX(w + Decode(template, 9, width))
   
   w = Decode(template, 5, width)     
   tmp = tmp + Rect(w, height) + MoveX(w + width*2)
   
   RETURN tmp
ENDPROC

PROC InitCode39(pitch, height)

   IF NOT ISASSIGNED(Code39) OR TYPE(Code39) <> "AY" THEN
      ARRAY Code39[44]
   ENDIF
   
   CodeList = " $%*+-./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"

   Code39[1]  = "010101000" ; 
   Code39[2]  = "000001110" ; $
   Code39[3]  = "000000111" ; %
   Code39[4]  = "001101000" ; *
   Code39[5]  = "000001011" ; +
   Code39[6]  = "000111000" ; -
   Code39[7]  = "100101000" ; .
   Code39[8]  = "000001101" ; /
   Code39[9]  = "001100100" ; 0
   Code39[10] = "100010100" ; 1
   Code39[11] = "010010100" ; 2
   Code39[12] = "110000100" ; 3
   Code39[13] = "001010100" ; 4
   Code39[14] = "101000100" ; 5
   Code39[15] = "011000100" ; 6
   Code39[16] = "000110100" ; 7
   Code39[17] = "100100100" ; 8
   Code39[18] = "010100100" ; 9
   Code39[19] = "100010010" ; A
   Code39[20] = "010010010" ; B
   Code39[21] = "110000010" ; C
   Code39[22] = "001010010" ; D
   Code39[23] = "101000010" ; E
   Code39[24] = "011000010" ; F
   Code39[25] = "000110010" ; G
   Code39[26] = "100100010" ; H
   Code39[27] = "010100010" ; I
   Code39[28] = "001100010" ; J
   Code39[29] = "100010001" ; K
   Code39[30] = "010010001" ; L
   Code39[31] = "110000001" ; M
   Code39[32] = "001010001" ; N
   Code39[33] = "101000001" ; O
   Code39[34] = "011000001" ; P
   Code39[35] = "000110001" ; Q
   Code39[36] = "100100001" ; R
   Code39[37] = "010100001" ; S
   Code39[38] = "001100001" ; T
   Code39[39] = "100011000" ; U
   Code39[40] = "010011000" ; V
   Code39[41] = "110001000" ; W
   Code39[42] = "001011000" ; X
   Code39[43] = "101001000" ; Y
   Code39[44] = "011001000" ; Z

   FOR i FROM 1 TO 44
      Code39[i] = CalcCode(Code39[i], pitch, height)
   ENDFOR
   
ENDPROC

; BuildBarcode() returns a string that, when sent to the printer,
; prints the barcode at that point and leaves the current point at the
; end of the barcode.

PROC BuildBarcode(src)
   PRIVATE
      i,
      j,
      indx,
      aBarcode

   j = LEN(src)
   aBarcode = ""
   
   FOR i FROM 1 TO j
   
      ; SEARCH is case insensitive, so lower case will be handled OK
      indx = SEARCH(SUBSTR(src, i, 1), CodeList)
      IF (indx > 0) THEN
         aBarcode = aBarcode + Code39[indx]
      ENDIF
   
   ENDFOR
   
   ; Add the start and stop characters (which are equal to '*')
   RETURN Code39[4] + aBarcode + Code39[4]
   
ENDPROC

; PrintBarcode() is an example that sends a barcode to the printer at
; the current point.  Unfortunately, this is a little more painful than
; it should be, because PRINT in Paradox 4.0 doesn't support memo
; variables.  It only prints the first 255 characters.

PROC PrintBarcode(txt)
   PRIVATE
      code,
      j,
      i
      
   code = BuildBarcode(txt)
   j = INT(LEN(code) / 200)
   FOR i FROM 0 TO j
      PRINT SUBSTR(code, 200*i + 1, 200)
   ENDFOR
ENDPROC

;Usage:
;
   InitCode39( 2.5, 200 )
   PrintBarcode( "TEST CODE" )
